Arc

Struct Arc 

pub struct Arc<S> {
    pub center: Point2D<S, UnknownUnit>,
    pub radii: Vector2D<S, UnknownUnit>,
    pub start_angle: Angle<S>,
    pub sweep_angle: Angle<S>,
    pub x_rotation: Angle<S>,
}
Available on crate feature geometry only.
An elliptic arc curve segment.

Fields

center: Point2D<S, UnknownUnit>radii: Vector2D<S, UnknownUnit>start_angle: Angle<S>sweep_angle: Angle<S>x_rotation: Angle<S>

Implementations



impl<S> Arc<S>
where S: Scalar,

pub fn cast<NewS>(self) -> Arc<NewS>
where NewS: NumCast,

Available on crate feature canvas only.

pub fn circle(center: Point2D<S, UnknownUnit>, radius: S) -> Arc<S>

Available on crate feature canvas only.

Create simple circle.

pub fn from_svg_arc(arc: &SvgArc<S>) -> Arc<S>

Available on crate feature canvas only.

Convert from the SVG arc notation.

pub fn to_svg_arc(&self) -> SvgArc<S>

Available on crate feature canvas only.

Convert to the SVG arc notation.

pub fn for_each_quadratic_bezier<F>(&self, cb: &mut F)

Available on crate feature canvas only.

Approximate the arc with a sequence of quadratic bézier curves.

pub fn for_each_quadratic_bezier_with_t<F>(&self, cb: &mut F)

Available on crate feature canvas only.

Approximate the arc with a sequence of quadratic bézier curves.

pub fn for_each_cubic_bezier<F>(&self, cb: &mut F)
where F: FnMut(&CubicBezierSegment<S>),

Available on crate feature canvas only.

Approximate the arc with a sequence of cubic bézier curves.

pub fn sample(&self, t: S) -> Point2D<S, UnknownUnit>

Available on crate feature canvas only.

Sample the curve at t (expecting t between 0 and 1).

pub fn x(&self, t: S) -> S

Available on crate feature canvas only.

pub fn y(&self, t: S) -> S

Available on crate feature canvas only.

pub fn sample_tangent(&self, t: S) -> Vector2D<S, UnknownUnit>

Available on crate feature canvas only.

Sample the curve’s tangent at t (expecting t between 0 and 1).

pub fn get_angle(&self, t: S) -> Angle<S>

Available on crate feature canvas only.

Sample the curve’s angle at t (expecting t between 0 and 1).

pub fn end_angle(&self) -> Angle<S>

Available on crate feature canvas only.

pub fn from(&self) -> Point2D<S, UnknownUnit>

Available on crate feature canvas only.

pub fn to(&self) -> Point2D<S, UnknownUnit>

Available on crate feature canvas only.

pub fn split_range(&self, t_range: Range<S>) -> Arc<S>

Available on crate feature canvas only.

Return the sub-curve inside a given range of t.

This is equivalent splitting at the range’s end points.

pub fn split(&self, t: S) -> (Arc<S>, Arc<S>)

Available on crate feature canvas only.

Split this curve into two sub-curves.

pub fn before_split(&self, t: S) -> Arc<S>

Available on crate feature canvas only.

Return the curve before the split point.

pub fn after_split(&self, t: S) -> Arc<S>

Available on crate feature canvas only.

Return the curve after the split point.

pub fn flip(&self) -> Arc<S>

Available on crate feature canvas only.

Swap the direction of the segment.

pub fn for_each_flattened<F>(&self, tolerance: S, callback: &mut F)
where F: FnMut(&LineSegment<S>),

Available on crate feature canvas only.

Approximates the curve with sequence of line segments.

The tolerance parameter defines the maximum distance between the curve and its approximation.

pub fn for_each_flattened_with_t<F>(&self, tolerance: S, callback: &mut F)
where F: FnMut(&LineSegment<S>, Range<S>),

Available on crate feature canvas only.

Approximates the curve with sequence of line segments.

The tolerance parameter defines the maximum distance between the curve and its approximation.

The end of the t parameter range at the final segment is guaranteed to be equal to 1.0.

pub fn flattened(&self, tolerance: S) -> Flattened<S>

Available on crate feature canvas only.

Returns the flattened representation of the curve as an iterator, starting after the current point.

pub fn fast_bounding_box(&self) -> Box2D<S, UnknownUnit>

Available on crate feature canvas only.

Returns a conservative rectangle that contains the curve.

pub fn bounding_box(&self) -> Box2D<S, UnknownUnit>

Available on crate feature canvas only.

Returns a conservative rectangle that contains the curve.

pub fn for_each_local_x_extremum_t<F>(&self, cb: &mut F)
where F: FnMut(S),

Available on crate feature canvas only.

pub fn for_each_local_y_extremum_t<F>(&self, cb: &mut F)
where F: FnMut(S),

Available on crate feature canvas only.

pub fn bounding_range_x(&self) -> (S, S)

Available on crate feature canvas only.

pub fn bounding_range_y(&self) -> (S, S)

Available on crate feature canvas only.

pub fn fast_bounding_range_x(&self) -> (S, S)

Available on crate feature canvas only.

pub fn fast_bounding_range_y(&self) -> (S, S)

Available on crate feature canvas only.

pub fn approximate_length(&self, tolerance: S) -> S

Available on crate feature canvas only.